require 'active_record'

module ArmaFixer
  module Validation
    module_function

    def run!
      validate_models_hash!
      validate_default_image_path!
      validate_attribute_checker!
    end

    private

    module_function

    def validate_models_hash!
      validate_models_hash_presence!
      validate_models_hash_contents!
    end

    def validate_models_hash_presence!
      return if defined?(ArmaFixer::MODELS_HASH) &&
                ArmaFixer::MODELS_HASH &&
                !ArmaFixer::MODELS_HASH.empty?

      fail 'ArmaFixer::MODELS_HASH cannot be undefined or empty.'
    end

    def validate_models_hash_contents!
      ArmaFixer::MODELS_HASH.each_value do |attributes|
        case attributes
        when Array
          if attributes.empty?
            fail 'attributes array cannot be empty'
          elsif !default_image_path_defined?
            fail 'ArmaFixer::DEFAULT_IMAGE_PATH must exist in case of array attributes'
          end
        when Hash
          if attributes.empty?
            fail 'attributes hash cannot be empty'
          else
            attributes.each_value do |path|
              fail "file not found at #{path}" unless File.exist?(path)
            end
          end
        else
          fail 'attributes must be in form of hash or array'
        end
      end
    end

    def validate_default_image_path!
      return unless default_image_path_defined?
      path = ArmaFixer::DEFAULT_IMAGE_PATH
      fail 'ArmaFixer::DEFAULT_IMAGE_PATH must be a string' unless path.is_a?(String)
      fail "file not found at '#{path}'" unless File.exist?(path)
    end

    def default_image
      ArmaFixer::DEFAULT_IMAGE_PATH if default_image_path_defined?
    end

    def default_image_path_defined?
      defined?(ArmaFixer::DEFAULT_IMAGE_PATH)
    end

    def validate_attribute_checker!
      if defined?(ArmaFixer::ATTR_CHECKER)
        unless ArmaFixer::ATTR_CHECKER.is_a?(Proc)
          fail 'ArmaFixer::ATTR_CHECKER must be a lambda'
        end
      end
    end
  end
end
